/*	$OpenBSD: hidccvar.h,v 1.1 2022/11/11 06:46:48 anton Exp $	*/

/*
 * Copyright (c) 2022 Anton Lindqvist <anton@openbsd.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>

struct hidcc_attach_arg {
	struct device	*device;
	void		*audio_cookie;

	void		*desc;
	int		 descsiz;
	int		 repid;
	int		 isize;

	int		 (*enable)(void *, int);
	void		*arg;
};

int		 hidcc_match(void *, int, uint8_t);
struct hidcc	*hidcc_attach(const struct hidcc_attach_arg *);
int		 hidcc_detach(struct hidcc *, int);
void		 hidcc_intr(struct hidcc *, void *, u_int);
