/*	$OpenBSD: onewiredevs.h,v 1.5 2008/02/23 23:21:19 miod Exp $	*/

/*
 * THIS FILE AUTOMATICALLY GENERATED.  DO NOT EDIT.
 *
 * Generated from:
 *	OpenBSD: onewiredevs,v 1.4 2008/02/23 23:20:52 miod Exp 
 */

#define ONEWIRE_FAMILY_DS1990	0x01
#define ONEWIRE_FAMILY_DS1991	0x02
#define ONEWIRE_FAMILY_DS1994	0x04
#define ONEWIRE_FAMILY_DS2405	0x05
#define ONEWIRE_FAMILY_DS1993	0x06
#define ONEWIRE_FAMILY_DS1992	0x08
#define ONEWIRE_FAMILY_DS1982	0x09
#define ONEWIRE_FAMILY_DS1995	0x0a
#define ONEWIRE_FAMILY_DS2505	0x0b
#define ONEWIRE_FAMILY_DS1996	0x0c
#define ONEWIRE_FAMILY_DS2506	0x0f
#define ONEWIRE_FAMILY_DS1920	0x10
#define ONEWIRE_FAMILY_DS2406	0x12
#define ONEWIRE_FAMILY_DS2430	0x14
#define ONEWIRE_FAMILY_DS195X	0x16
#define ONEWIRE_FAMILY_DS28E04	0x1c
#define ONEWIRE_FAMILY_DS2423	0x1d
#define ONEWIRE_FAMILY_DS2409	0x1f
#define ONEWIRE_FAMILY_DS1822	0x22
#define ONEWIRE_FAMILY_DS2433	0x23
#define ONEWIRE_FAMILY_DS2415	0x24
#define ONEWIRE_FAMILY_DS2438	0x26
#define ONEWIRE_FAMILY_DS2417	0x27
#define ONEWIRE_FAMILY_DS18B20	0x28
#define ONEWIRE_FAMILY_DS2408	0x29
#define ONEWIRE_FAMILY_DS2890	0x2c
#define ONEWIRE_FAMILY_DS2431	0x2d
#define ONEWIRE_FAMILY_DS2760	0x30
#define ONEWIRE_FAMILY_DS2413	0x3a
#define ONEWIRE_FAMILY_DS2422	0x41
#define ONEWIRE_FAMILY_DS28EA00	0x42
#define ONEWIRE_FAMILY_DS28EC20	0x43
